% \VAR{ type }
\node[\VAR{ node_options }] (\VAR{ block.id })
\BLOCK{ if block.parent is not none }
[right=\VAR{ block.arrow_length + block.shift[0] }cm of \VAR{ block.parent.last.id }, yshift=\VAR{ block.shift[1] }cm]
\BLOCK{ endif }
{\VAR{ block.text }};
\BLOCK{ if block.parent is not none }
\draw[\VAR{ block.arrow_options }, rectangle connector=\VAR{ connector_position }cm] (\VAR{ block.parent.last.id }.east) to (\VAR{ block.id }.west);
\BLOCK{ endif }
